/*
 * Copyright 1999 ValiCert Inc. All Rights Reserved.
 *
 * $Id: vtk_cert.h,v 1.10 1999/12/18 09:17:26 jackk Exp $
 *
 * ValiCert Validator Toolkit
 *
 * Certificate manipulation/encapsulation functions.
 */

#ifndef VTK_CERT_H_
#define VTK_CERT_H_

#include <vtk_defs.h>

#ifdef __cplusplus
extern "C" {
#endif


/*
 * Certificate encapsulation functions
 */

/*
 * Vtk_CertNew
 *
 * This call allocates memory for an empty Vtk_Cert structure.
 */
VTK_FUNC Vtk_Cert* VTK_CALL Vtk_CertNew(const Vtk_Ctxt *ctxt);


/*
 * Vtk_CertDelete
 *
 * This call releases memory occupied by a Vtk_Cert structure.
 * All certificates allocated with Vtk_CertNew need to
 * be released using this call.
 */
VTK_FUNC void VTK_CALL Vtk_CertDelete(Vtk_Cert *cert);


/*
 * Vtk_CertInit
 *
 * Initializes a Vtk_Cert data structure from supplied data.
 * The certificate data can be in DER or BASE64 formats.
 * A single Vtk_Cert can only be initialized once.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_CertInit(const Vtk_Ctxt* ctxt, Vtk_Cert *dest, const Vtk_Buffer *source);


/*
 * Vtk_CertLoadFromFile
 *
 * This call reads certificate information from a supplied file.
 * The certificate data can be either in DER or BASE64 format.
 * A single Vtk_Cert can only be loaded once.
*/
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_CertLoadFromFile(const Vtk_Ctxt* ctxt, Vtk_Cert *into,
	const char *fileName, enum Vtk_DataFormat format);


/*
 * Vtk_CertGetInfo
 *
 * This call obtains detailed certificate information. The returned
 * Vtk_CertInfo structure needs to be released using the
 * Vtk_CertInfoDelete call.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_CertGetInfo(const Vtk_Ctxt* ctxt, const Vtk_Cert *from, Vtk_CertInfo **dest);


/*
 * Vtk_CertInfoDelete
 *
 * This call releases memory from the Vtk_CertInfo structure.
 */
VTK_FUNC Vtk_uint32 VTK_CALL Vtk_CertInfoDelete(Vtk_CertInfo *ci);


/*
 * Vtk_CertGetIssuer
 *
 * This call searches the ctxt's CA certificates for the issuer of
 * the speicifed certificate.
 * The obtained certificate needs to be deleted using Vtk_CertDelete.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_CertGetIssuer(const Vtk_Ctxt *ctxt, const Vtk_Cert *cert, Vtk_Cert **issuer);


/*
 * Vtk_ExtensionNew
 *
 * This call allocates memory for an empty Vtk_Extension structure.
 */
VTK_FUNC Vtk_Extension* VTK_CALL Vtk_ExtensionNew(const Vtk_Ctxt* ctxt);


/*
 * Vtk_ExtensionInit
 *
 * Initializes a Vtk_Extension data structure from supplied data.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_ExtensionInit(const Vtk_Ctxt* ctxt, Vtk_Extension *ext, const Vtk_Buffer *oid,
		int critical, const Vtk_Buffer *value);

/*
 * Vtk_CertGetExtensions
 *
 * This call returns a list of certificate extensions. The returned value
 * needs to be released using the Vtk_ExtensionsDelete call.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_CertGetExtensions(const Vtk_Ctxt* ctxt, const Vtk_Cert *from, Vtk_Extensions **into);


/*
 * Extension manipulation functions
 *
 * The extension structure encapsulates any X.509 extension.
 * This includes not only X.509 certificate extensions, but
 * also OCSP, CRT, and CRL extensions.
 */
/*
 * Vtk_ExtensionsDelete
 *
 * This call releases a Vtk_Exensions structure.
 */
VTK_FUNC void VTK_CALL
Vtk_ExtensionsDelete(Vtk_Extensions *ext);


/*
 * Vtk_ExtensionGetByOid
 *
 * This call searches the list of extensions by OID. The oid can be
 * specified in dot notation e.g. "1.2.3.4."
 * The returned Vtk_Extension structure should be released by
 * calling Vtk_ExtensionDelete.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_ExtensionGetByOid(const Vtk_Ctxt *ctxt, const Vtk_Extensions *exts,
	const Vtk_Buffer *oid, Vtk_Extension **dest);


/*
 * Vtk_ExtensionsGetCount
 *
 * This call returns the number of extensions present.
 */
VTK_FUNC int VTK_CALL
Vtk_ExtensionsGetCount(const Vtk_Ctxt *ctxt, const Vtk_Extensions *exts);


/*
 * Vtk_ExtensionsGetith
 *
 * This call returns the ith extension from the extension list.
 * The index is 0 based. The total number of extensions can be
 * determined by calling Vtk_ExtenionsGetCount().
 * The returned Vtk_Extension structure should be released by
 * calling Vtk_ExtensionDelete.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_ExtensionsGetith(const Vtk_Ctxt *ctxt, const Vtk_Extensions *exts,
	int i, Vtk_Extension **dest);


/*
 * Vtk_ExtensionDelete
 *
 * This call deletes a previously obtained Vtk_Extension
 * structure from the toolkit.
 */
VTK_FUNC void VTK_CALL
Vtk_ExtensionDelete(Vtk_Extension *ext);


/*
 * Certificate Store functions.
 *
 * This structure is used as a container for multiple certificates.
 */
/*
 * Vtk_CertStoreNew
 *
 * This call creates a new empty Vtk_CertStore structure. Use
 * Vtk_CertStoreDelete to release the memory and resources allocated.
 */
VTK_FUNC Vtk_CertStore* VTK_CALL Vtk_CertStoreNew(const Vtk_Ctxt *ctxt);


/*
 * Vtk_CertStoreDelete
 *
 * This call frees memory and resources previously allocated by
 * Vtk_CertStoreNew.
 */
VTK_FUNC void VTK_CALL Vtk_CertStoreDelete(Vtk_CertStore *toFree);


/*
 * Vtk_CertStoreLoadFromFile
 *
 * This call reads certificates from the specified file and adds them
 * to the supplied Vtk_CertStore. The certificates can be
 * either in BASE64 or DER format.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_CertStoreLoadFromFile(const Vtk_Ctxt *ctxt, Vtk_CertStore *dest,
	const char *fileName, enum Vtk_DataFormat format);


/*
 * Vtk_CertStoreAddCert
 *
 * This call adds a single certificate to the supplied Vtk_CertStore.
 * The certificate is copied, so the caller must call Vtk_CertDelete.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_CertStoreAddCert(const Vtk_Ctxt *ctxt, Vtk_CertStore *dest, const Vtk_Cert *aCert);


/*
 * Vtk_CertStoreAddCertRaw
 *
 * This call is a convenience function to add a raw certificate
 * (DER or BASE64 format) to a Vtk_CertStore.
 */
VTK_FUNC Vtk_uint32 VTK_CALL
Vtk_CertStoreAddCertRaw(const Vtk_Ctxt *ctxt, Vtk_CertStore *dest,
	const Vtk_Buffer *certData);



#ifdef __cplusplus
}
#endif

#endif /* VTK_CERT_H_ */

