// Certificate.cpp: implementation of the CCertificate class.
//
//////////////////////////////////////////////////////////////////////

#include "Certificate.h"
#include "VString.h"
#include "CertificateInfo.h"
#include "Extension.h"
#include "ValRespDetails.h"
#include "ValRespSingleCertDetails.h"
#include <cassert>
#include <vtk_cert.h>
#include <vtk_error.h>
#include <vtk_valid.h>  // Vtk_CRLValidateCert

/*
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif
*/

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


CCertificate::CCertificate():m_pCert(NULL)
{
}


CCertificate::CCertificate(const CContext& context, const VString& source)
{
	assert(context.GetVtkCtxt());
	assert(source.size());

	// Copy certificate from string into the buffer
	Vtk_Buffer buffer;
	buffer.type = VTK_DF_BASE64;
	buffer.dPtr = const_cast<unsigned char*>((const unsigned char *) source.c_str());
	buffer.len  = source.size();

	// Allocate space for this certificate
	m_pCert = Vtk_CertNew(context.GetVtkCtxt());
	if (m_pCert == NULL)
		throw VString((const char *)"Certificate initialization failed.");


	// Init certificate from Vtk_Buffer
	Vtk_uint32 nRet = Vtk_CertInit(context.GetVtkCtxt(), m_pCert, &buffer);
	if (nRet != VTK_OK)
	{
		Vtk_CertDelete(m_pCert);

		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}


CCertificate::CCertificate(const CContext& context, const unsigned char *pSource, int size,
		                   enum Vtk_DataFormat format)
{
	assert(context.GetVtkCtxt());
	assert(pSource);

	// Copy input parameters into a Vtk_Buffer
	Vtk_Buffer buffer;
	buffer.dPtr = const_cast<unsigned char*>(pSource);
	buffer.len  = size;
	buffer.type = format;

	// Allocate space for this certificate
	m_pCert = Vtk_CertNew(context.GetVtkCtxt());
	if (m_pCert == NULL)
		throw VString((const char *)"Certificate initialization failed.");

	// Init certificate from Vtk_Buffer
	Vtk_uint32 nRet = Vtk_CertInit(context.GetVtkCtxt(), m_pCert, &buffer);
	if (nRet != VTK_OK)
	{
		Vtk_CertDelete(m_pCert);

		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}



CCertificate::CCertificate(const CContext& context, const VString& fileName, 
						   enum Vtk_DataFormat format)
{
	assert(context.GetVtkCtxt());
  
	// Allocate space for this certificate
	m_pCert = Vtk_CertNew(context.GetVtkCtxt());
	if (m_pCert == NULL)
		throw VString("Certificate initialization failed.");

	// Init certificate from File
	Vtk_uint32 nRet = Vtk_CertLoadFromFile(context.GetVtkCtxt(), m_pCert, 
		                                   fileName.c_str(), format);
	if (nRet != VTK_OK)
	{
		Vtk_CertDelete(m_pCert);

		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}


CCertificate::CCertificate(const CCertificate& obj)
{
	*this = obj;
}

CCertificate& CCertificate::operator=(const CCertificate& obj)
{
	if (this != &obj)
	{
		// BUGBUG
		// Temporary the assignment is not allowed !
		// Support from Toolkit Validator is needed. Possible solutions:
		//  - copy Vtk_Cert structure
		//  - increase reference counter in for Vtk_Cert structure of obj

		// this->m_pCert = obj.m_pCert;
		throw VString("Assignment of CCertificate is not supported.");
	}
	return *this;
}



CCertificate::~CCertificate()
{
	if (m_pCert)
		Vtk_CertDelete(m_pCert);
}



// Get certificate information
void CCertificate::GetCertInfo(const CContext& context, CCertificateInfo& certInfo) const
{
	assert(context.GetVtkCtxt());
	assert(this->m_pCert);

	// Get Certificate info
	Vtk_uint32 nRet = Vtk_CertGetInfo(context.GetVtkCtxt(), m_pCert, &(certInfo.m_pCertInfo));
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}



void CCertificate::GetIssuerCert(const CContext& context, CCertificate &issuerCert) const
{
	assert(context.GetVtkCtxt());
	assert(this->m_pCert);

	// Get issuer certificate
	Vtk_Cert *issuer;
	Vtk_uint32 nRet = Vtk_CertGetIssuer(context.GetVtkCtxt(), m_pCert, &issuer);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}

	issuerCert.m_pCert = issuer;	
}



void CCertificate::GetExtensions(const CContext& context, CExtensions& extsOut) const
{
	assert(context.GetVtkCtxt());
	assert(this->m_pCert);

	// Get certificate extensions

	Vtk_Extensions *pExts;
	Vtk_uint32 nRet = Vtk_CertGetExtensions(context.GetVtkCtxt(), m_pCert, &pExts);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}

	extsOut.m_pExts = pExts;	
}


void CCertificate::CRLValidate(const CContext& context, Vtk_uint32 *pStatus,
							   CValRespDetails *pRespDetails /*= NULL*/,
							   CValRespSingleCertDetails *pCertDetails/*=NULL*/) const
{
	assert(context.GetVtkCtxt());
	assert(this->m_pCert);

	// Get Vtk_ValRespDetails for Vtk_CRLValidateCert (if not Null)
	Vtk_ValRespDetails **respDetails = NULL;
	if (pRespDetails)
		respDetails = &(pRespDetails->m_pRespDetails);

	// Get Vtk_ValRespDetails for Vtk_CRLValidateCert (if not Null)
	Vtk_ValRespSingleCertDetails **certDetails = NULL;
	if (pCertDetails)
		certDetails = &(pCertDetails->m_pCertDetails);

	// Make CRL validation
	Vtk_uint32 nRet = Vtk_CRLValidateCert(context.GetVtkCtxt(), m_pCert, pStatus,
	                                      respDetails, certDetails);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}
