// Context.cpp: implementation of the CContext class.
//
//////////////////////////////////////////////////////////////////////

#include "Context.h"
#include "VString.h"
#include "Certificate.h"
#include "CertificateStore.h"
#include "ProtocolDetails.h"
#include <vtk_error.h>
#include <cassert>


/*
#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif
*/

int CContext::m_refCount = 0;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CContext::CContext()
{
	if (m_refCount == 0)
	{
		// Initialize Toolkit 
		Vtk_uint32 nRet = Vtk_Init();
		if (nRet != VTK_OK)
		{
			VString error(Vtk_ErrorToString(nRet));
			throw error;
		}
	}

	// Allocate space for this context structure
	m_pCtxt = Vtk_CtxtNew();
	if (m_pCtxt == NULL)
		throw VString("Context initialization failed.");

	// Incremenr reference counter
	m_refCount++;
}


CContext::CContext(const CContext& obj)
{
	*this = obj;
}


CContext& CContext::operator=(const CContext& obj)
{
	if (this != &obj)
	{
		// BUGBUG
		// Simple pointer assignment is only temporary solution !
		// Support from Toolkit Validator is needed. Possible solutions:
		//  - copy Vtk_Ctxt structure
		//  - increase reference counter in for Vtk_Ctxt structure of obj

		// this->m_pCtxt = obj.m_pCtxt;
		// m_refCount++;

		throw VString("Assignment of CContext is not supported.");
	}
	return *this;
}


CContext::~CContext()
{
	m_refCount--;
	if (m_refCount == 0)
	{
		// Release Toolkit 
		Vtk_uint32 nRet = Vtk_Finish();
		if (nRet != VTK_OK)
		{
			VString error(Vtk_ErrorToString(nRet));
			throw error;
		}
	}

	// Release this context
	if (m_pCtxt)
		Vtk_CtxtDelete(m_pCtxt);
}


void CContext::SetDefaultVa(Vtk_ValidationMech valMech, const VString& vaUrl)
{
	assert(this->m_pCtxt);

	if (valMech != VTK_VM_CRT && valMech != VTK_VM_OCSP)
		throw VString("Unsupported validation mechanism.");

	// Set Validation mechanism
	Vtk_uint32 nRet = Vtk_CtxtSetDefaultVa(this->m_pCtxt, vaUrl.c_str(), valMech);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}


void CContext::AddCertificate(const CCertificate& cert, int type)
{
	assert(cert.GetVtkCert());
	assert(this->m_pCtxt);

	// Add individual certificate to the list of trusted certificates
	Vtk_uint32 nRet = Vtk_CtxtAddCert(this->m_pCtxt, type, cert.GetVtkCert());
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}


void CContext::AddCertificates(const CCertificateStore& certStore, int type)
{
	assert(certStore.GetVtkCertStore());
	assert(this->m_pCtxt);

	// Add list of certificates to the list of trusted certificates
	Vtk_uint32 nRet = Vtk_CtxtAddCerts(this->m_pCtxt, type, certStore.GetVtkCertStore());
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}


void CContext::SetVaInfo(const CCertificate& cert, const VString& vaUrl, 
						 enum Vtk_ValidationMech mech, 
						 const CProtocolDetails* pDetails /*=NULL*/,  
						 const CCertificateStore* pCertStore /*=NULL*/)
{
	assert(cert.GetVtkCert());
	assert(this->m_pCtxt);
	if (pCertStore)
		assert(pCertStore->GetVtkCertStore());

	// Get Vtk structures for Protocol Details (if not Null)
	const Vtk_ProtocolDetails *pVtkDetails = NULL;
	if (pDetails)
		pVtkDetails = &(pDetails->GetVtkDetails());

	// Get Vtk structures for Certificate Store (if not Null)
	const Vtk_CertStore *pVtkCertStore = NULL;
	if (pCertStore)
		pVtkCertStore = pCertStore->GetVtkCertStore();
	

	// Set VA info
	Vtk_uint32 nRet = Vtk_CtxtSetVaInfo(m_pCtxt, cert.GetVtkCert(), vaUrl.c_str(), 
		                                mech, pVtkDetails, pVtkCertStore);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}



// Accessors and Mutators for Context Options
/////////////////////////////////////////////////////////////////////

void CContext::SetHttpProxy(const VString& host, int port)
{
	assert(this->m_pCtxt);

	// Set option type (key)
	Vtk_CtxtOption ctxtOpt;
	ctxtOpt.option = CO_HTTPProxy;

	// Set option values
	Vtk_ProxyInfo proxyInfo;
	proxyInfo.host = const_cast <char*>(host.c_str());
	proxyInfo.port = port;
	ctxtOpt.d.aProxyInfo = &proxyInfo; 

	Vtk_uint32 nRet = Vtk_CtxtSetOption(m_pCtxt, &ctxtOpt);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}

void CContext::GetHttpProxy(VString& host, int& port) const
{
	assert(this->m_pCtxt);

	// Set option type (key)
	Vtk_CtxtOption ctxtOpt;
	ctxtOpt.option = CO_HTTPProxy;

	// Get required option from context
	Vtk_uint32 nRet = Vtk_CtxtGetOption(m_pCtxt, &ctxtOpt);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}

	// Get returned option values
	host = ctxtOpt.d.aProxyInfo->host;
	port = ctxtOpt.d.aProxyInfo->port;

	// Free returned context option
	Vtk_CtxtOptionDeleteContent(&ctxtOpt);
}


void CContext::SetIntOption(enum Vtk_CtxtOptionType type, int val)
{
	assert(this->m_pCtxt);
	assert(type == CO_ServiceLocatorExt || type == CO_ClientInfoExt || 
		   type == CO_OCSPNonceExt || type == CO_UseAIAData || type == CO_LoadLDAPLib ||       type == CO_MaxTimeSkew ||
		   type == CO_MaxCrlCacheTime || type == CO_CrlNoNextUpdateCacheTime ||
		   type == CO_LDAPSearchTimeOut || type == CO_MaxTimeSkew);

	// Set option type (key)
	Vtk_CtxtOption ctxtOpt;
	ctxtOpt.option = type;

	// Set option values
	ctxtOpt.d.anInt = val;

	Vtk_uint32 nRet = Vtk_CtxtSetOption(m_pCtxt, &ctxtOpt);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}

int CContext::GetIntOption(enum Vtk_CtxtOptionType type) const
{
	assert(this->m_pCtxt);
	assert(type == CO_ServiceLocatorExt || type == CO_ClientInfoExt || 
		   type == CO_OCSPNonceExt || type == CO_UseAIAData || type == CO_LoadLDAPLib || 
		   type == CO_MaxTimeSkew || type == CO_MaxCrlCacheTime || 
		   type == CO_CrlNoNextUpdateCacheTime || type == CO_LDAPSearchTimeOut || 
		   type == CO_MaxTimeSkew);

	
	// Set option type (key)
	Vtk_CtxtOption ctxtOpt;
	ctxtOpt.option = type;

	// Get required option from context
	Vtk_uint32 nRet = Vtk_CtxtGetOption(m_pCtxt, &ctxtOpt);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}

	// Free returned context option
	Vtk_CtxtOptionDeleteContent(&ctxtOpt);

	return ctxtOpt.d.anInt;
}



void CContext::SetServiceLocExt(bool bVal)
{
	int val = bVal;

	SetIntOption(CO_ServiceLocatorExt, val);
}

bool CContext::GetServiceLocExt() const
{
	int val = GetIntOption(CO_ServiceLocatorExt);

	bool bVal = false;
	if (val)
		bVal = true;

	return bVal;
}



void CContext::SetClientInfExt(bool bVal)
{
	int val = bVal;

	SetIntOption(CO_ClientInfoExt, val);
}

bool CContext::GetClientInfExt() const
{
	int val = GetIntOption(CO_ClientInfoExt);

	bool bVal = false;
	if (val)
		bVal = true;

	return bVal;
}


void CContext::SetOscpNonceExt(bool bVal)
{
	int val = bVal;

	SetIntOption(CO_OCSPNonceExt, val);
}

bool CContext::GetOscpNonceExt() const
{
	int val = GetIntOption(CO_OCSPNonceExt);

	bool bVal = false;
	if (val)
		bVal = true;

	return bVal;
}


void CContext::SetMaxTimeSkew(int val)
{
	SetIntOption(CO_MaxTimeSkew, val);
}

int CContext::GetMaxTimeSkew() const
{
	return GetIntOption(CO_MaxTimeSkew);
}


void CContext::SetUseAiaData(bool bVal)
{
	int val = (int) bVal;
	SetIntOption(CO_UseAIAData, val);
}

bool CContext::GetUseAiaData() const
{
	int val = GetIntOption(CO_UseAIAData);

	bool bVal = false;
	if (val)
		bVal = true;

	return bVal;
}


void CContext::SetLoadLDAPLib(bool bVal)
{
	int val = (int) bVal;
	SetIntOption(CO_LoadLDAPLib, val);
}

bool CContext::GetLoadLDAPLib() const
{
	int val = GetIntOption(CO_LoadLDAPLib);

	bool bVal = false;
	if (val)
		bVal = true;

	return bVal;
}

void CContext::SetCrlCacheDir(const VString& dir)
{
	assert(this->m_pCtxt);

	// Set option type (key)
	Vtk_CtxtOption ctxtOpt;
	ctxtOpt.option = CO_CrlCacheDir;

	// Set option values
	ctxtOpt.d.aChar = const_cast <char*>(dir.c_str());

	Vtk_uint32 nRet = Vtk_CtxtSetOption(m_pCtxt, &ctxtOpt);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}

void CContext::GetCrlCacheDir(VString& dir) const
{
	assert(this->m_pCtxt);

	// Set option type (key)
	Vtk_CtxtOption ctxtOpt;
	ctxtOpt.option = CO_CrlCacheDir;

	// Get required option from context
	Vtk_uint32 nRet = Vtk_CtxtGetOption(m_pCtxt, &ctxtOpt);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}

	// Get returned option values
	dir = ctxtOpt.d.aChar;

	// Free returned context option
	Vtk_CtxtOptionDeleteContent(&ctxtOpt);
}


void CContext::SetMaxCrlCacheTime(int val)
{
	SetIntOption(CO_MaxCrlCacheTime, val);
}

int CContext::GetMaxCrlCacheTime() const
{
	return GetIntOption(CO_MaxCrlCacheTime);
}


void CContext::SetCrlNoNextUpdateCacheTime(int val)
{
	SetIntOption(CO_CrlNoNextUpdateCacheTime, val);
}

int CContext::GetCrlNoNextUpdateCacheTime() const
{
	return GetIntOption(CO_CrlNoNextUpdateCacheTime);
}



void CContext::SetLdapSearchTimeOut(int val)
{
	SetIntOption(CO_LDAPSearchTimeOut, val);
}

int CContext::GetLdapSearchTimeOut() const
{
	return GetIntOption(CO_LDAPSearchTimeOut);
}



void CContext::SetDelegatedCertCb(Vtk_DelegatedCertCallback callback, void* userHdl)
{
	assert(this->m_pCtxt);

	// Set option type (key)
	Vtk_CtxtOption ctxtOpt;
	ctxtOpt.option = CO_DelegatedCertCB;

	// Set option values
	Vtk_Callback structCB;
	structCB.f.DelCertCB = callback;
	structCB.userHdl = userHdl;
	ctxtOpt.d.aCB = &structCB;

	Vtk_uint32 nRet = Vtk_CtxtSetOption(m_pCtxt, &ctxtOpt);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}

void CContext::GetDelegatedCertCb(Vtk_DelegatedCertCallback& callback, void*& userHdl) const
{
	assert(this->m_pCtxt);

	// Set option type (key)
	Vtk_CtxtOption ctxtOpt;
	ctxtOpt.option = CO_DelegatedCertCB;

	// Get required option from context
	Vtk_uint32 nRet = Vtk_CtxtGetOption(m_pCtxt, &ctxtOpt);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}

	// Get returned option values
	callback = ctxtOpt.d.aCB->f.DelCertCB;
	userHdl     = ctxtOpt.d.aCB->userHdl;

	// Free returned context option
	Vtk_CtxtOptionDeleteContent(&ctxtOpt);
}


void CContext::SetOcspSignInfo(Vtk_OCSPSignCallback callback, void* userHdl, 
							   const CCertificate& signerCert)
{
	assert(this->m_pCtxt);

	// Set option type (key)
	Vtk_CtxtOption ctxtOpt;
	ctxtOpt.option = CO_OCSPSignInfo;

	// Set option values
	Vtk_OCSPSignInfo signInfo;
	signInfo.OCSPSignCB = callback;
	signInfo.userHdl = userHdl;
	signInfo.signerCert = const_cast <Vtk_Cert*>(signerCert.GetVtkCert());

	ctxtOpt.d.aSignInfo = &signInfo;

	Vtk_uint32 nRet = Vtk_CtxtSetOption(m_pCtxt, &ctxtOpt);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}


void CContext::GetOcspSignInfo(Vtk_OCSPSignCallback& callback, void*& userHdl, 
							   CCertificate& signerCert) const
{
	assert(this->m_pCtxt);

	// Set option type (key)
	Vtk_CtxtOption ctxtOpt;
	ctxtOpt.option = CO_OCSPSignInfo;

	// Get required option from context
	Vtk_uint32 nRet = Vtk_CtxtGetOption(m_pCtxt, &ctxtOpt);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}

	// Get returned option values
	callback = ctxtOpt.d.aSignInfo->OCSPSignCB;
	userHdl = ctxtOpt.d.aSignInfo->userHdl;

	// Copy pointer to returned certificate from Vtk_CtxtOption
	// then set pointer in Vtk_CtxtOption to NULL -> so it will
	// not be deleted in following Vtk_CtxtOptionDeleteContent()
	signerCert.m_pCert = ctxtOpt.d.aSignInfo->signerCert;
	ctxtOpt.d.aSignInfo->signerCert = NULL;

	// Free returned context option
	Vtk_CtxtOptionDeleteContent(&ctxtOpt);
}



