// Log.cpp: implementation of the CLog class.
//
//////////////////////////////////////////////////////////////////////

#include "Log.h"
#include "Context.h"
#include <cassert>
#include <vtk_error.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


CLog::CLog()
{
	/* Set default logging options */
	this->m_logOptions.logLevel = LOG_Error;
	this->m_logOptions.logMode = VTK_LOG_MODE_DEFAULT;
	this->m_logOptions.logFileName.type = VTK_DF_STRING;
	this->m_logOptions.logFileName.dPtr = NULL;
	this->m_logOptions.logFileName.len  = 0;
	this->m_logOptions.openLogCB  = NULL;
	this->m_logOptions.closeLogCB = NULL;
	this->m_logOptions.writeLogCB = NULL;
}


CLog::CLog(enum Vtk_CtxtLogType logLevel, 
           bool overwriteLogFile /* = false */,
           bool flushEachMessage /* = false */,
           const VString& logFileName /* = VString() */, 
           Vtk_OpenLogCallback openLogCB   /* = NULL */,
           Vtk_CloseLogCallback closeLogCB /* = NULL */,
           Vtk_WriteLogCallback writeLogCB /* = NULL */)
{
	this->m_logOptions.logLevel = logLevel;
	if (overwriteLogFile)
		this->m_logOptions.logMode |= VTK_LOG_MODE_OVERWRITE;
	if (flushEachMessage)
		this->m_logOptions.logMode |= VTK_LOG_MODE_FLUSH;

	// Log File Name 
	this->m_logOptions.logFileName.type = VTK_DF_STRING;
	if (logFileName.size())
	{
		this->m_logOptions.logFileName.dPtr = (unsigned char*) 
			                                  (new char[logFileName.size() + 1]);
		if (this->m_logOptions.logFileName.dPtr == NULL)
			throw VString((const char *)"Memory alocation failed.");

		strcpy((char *)this->m_logOptions.logFileName.dPtr, logFileName.c_str());
		this->m_logOptions.logFileName.len = logFileName.size();
	}
	else
	{
		this->m_logOptions.logFileName.dPtr = NULL;
		this->m_logOptions.logFileName.len  = 0;
	}


	if (openLogCB && closeLogCB && writeLogCB)
	{
		this->m_logOptions.openLogCB = openLogCB;
		this->m_logOptions.closeLogCB = closeLogCB;
		this->m_logOptions.writeLogCB = writeLogCB;
	}
	else
	{
		this->m_logOptions.openLogCB = NULL;
		this->m_logOptions.closeLogCB = NULL;
		this->m_logOptions.writeLogCB = NULL;
	}
}


CLog::~CLog()
{
	// Delete memory allocated for logging file name
	if (this->m_logOptions.logFileName.dPtr)
		delete[] this->m_logOptions.logFileName.dPtr;
}



CLog::CLog(const CLog& obj)
{
	this->m_logOptions.logFileName.dPtr = NULL;
	*this = obj;
}


CLog& CLog::operator=(const CLog& obj)
{
	if (this != &obj)
	{
		this->m_logOptions.logLevel = obj.m_logOptions.logLevel;
		this->m_logOptions.logMode  = obj.m_logOptions.logMode;

		// Log File Name 
		assert(obj.m_logOptions.logFileName.type == VTK_DF_STRING);
		this->m_logOptions.logFileName.type = obj.m_logOptions.logFileName.type;

		// Release old allocated memory
		if (this->m_logOptions.logFileName.dPtr)
		{
			delete[] this->m_logOptions.logFileName.dPtr;
			this->m_logOptions.logFileName.dPtr = NULL;
		}

		if (obj.m_logOptions.logFileName.dPtr)
		{
			// Allocate new memory and copy the filename
			this->m_logOptions.logFileName.dPtr = (unsigned char *)
				    new char[strlen((const char *)obj.m_logOptions.logFileName.dPtr) + 1];
			if (this->m_logOptions.logFileName.dPtr == NULL)
				throw VString((const char *)"Memory alocation failed.");

			strcpy((char *)this->m_logOptions.logFileName.dPtr, 
				   (const char *)obj.m_logOptions.logFileName.dPtr);
			this->m_logOptions.logFileName.len = 
				    strlen((const char *)obj.m_logOptions.logFileName.dPtr);
		}

		this->m_logOptions.openLogCB  = obj.m_logOptions.openLogCB;
		this->m_logOptions.closeLogCB = obj.m_logOptions.closeLogCB;
		this->m_logOptions.writeLogCB = obj.m_logOptions.writeLogCB;
	}
	return *this;
}


// Main Log Functions
void CLog::OpenLog(CContext& context)
{
	assert(context.GetVtkCtxt());
	Vtk_uint32 nRet = Vtk_OpenLog(const_cast<Vtk_Ctxt*>(context.GetVtkCtxt()), &m_logOptions);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}


void CLog::CloseLog(CContext& context)
{
	assert(context.GetVtkCtxt());
	Vtk_CloseLog(const_cast<Vtk_Ctxt*>(context.GetVtkCtxt()));
}


void CLog::WriteLog(const CContext& context, enum Vtk_CtxtLogType type, const VString& msg)
{
	assert(context.GetVtkCtxt());
	Vtk_WriteLog(context.GetVtkCtxt(), type, msg.c_str());
}
