// Validator.cpp: implementation of the CValidator class.
//
//////////////////////////////////////////////////////////////////////

#include "Validator.h"
#include "VString.h"
#include "Context.h"
#include "Certificate.h"
#include "CertificateStore.h"
#include "ValRespDetails.h"
#include "ValRespSingleCertDetails.h"
#include "ValQueries.h"
#include "Extension.h"
#include <cassert>
#include <vtk_valid.h>
#include <vtk_error.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CValidator::CValidator():m_pValidation(NULL)
{
}


CValidator::CValidator(const CContext& context)
{
	assert(context.GetVtkCtxt());

	// Allocate space for this Validation structure
	m_pValidation = Vtk_ValidationNew(context.GetVtkCtxt());
	if (m_pValidation == NULL)
		throw VString("Validation initialization failed.");
}


CValidator::CValidator(const CValidator& obj)
{
	*this = obj;
}


CValidator& CValidator::operator=(const CValidator& obj)
{
	if (this != &obj)
	{
		// BUGBUG
		// Simple pointer assignment is only temporary solution !
		// Support from Toolkit Validator is needed. 

		throw VString("Assignment of CValidator is not supported.");
	}
	return *this;
}



CValidator::~CValidator()
{
	if (m_pValidation)
		Vtk_ValidationDelete(m_pValidation);
}


void CValidator::AddCertificate(const CContext& context, const CCertificate& cert,
								const CCertificate& issuerCert, CValHdl *pHdl/*=NULL*/)
{
	assert(context.GetVtkCtxt());
	assert(cert.GetVtkCert());
	assert(issuerCert.GetVtkCert());
	assert(m_pValidation);

	// Get Vtk_ValHdl (if not Null)
	Vtk_ValHdl **hdl = NULL;
	if (pHdl)
		hdl = &(pHdl->m_pHdl);

	// Add individual certificate to Validation query
	Vtk_uint32 nRet = Vtk_ValidationAddCert(context.GetVtkCtxt(), m_pValidation, 
		                                    cert.GetVtkCert(), issuerCert.GetVtkCert(),
											hdl);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}



void CValidator::AddCertificate(const CContext& context, const unsigned char *pCertSource, 
		                int certSize, enum Vtk_DataFormat certFormat, 
		                const unsigned char *pIssuerCertSource, int issuerCertSize, 
						enum Vtk_DataFormat issuerCertFormat, CValHdl *pHdl/*=NULL*/)
{
	assert(context.GetVtkCtxt());
	assert(pCertSource);
	assert(pIssuerCertSource);
	assert(m_pValidation);

	// Copy input parameters into a Vtk_Buffer (for user certificate and issuer certificate)
	Vtk_Buffer certBuffer;
	certBuffer.dPtr = const_cast<unsigned char*>(pCertSource);
	certBuffer.len  = certSize;
	certBuffer.type = certFormat;

	Vtk_Buffer issuerCertBuffer;
	issuerCertBuffer.dPtr = const_cast<unsigned char*>(pIssuerCertSource);
	issuerCertBuffer.len  = issuerCertSize;
	issuerCertBuffer.type = issuerCertFormat;


	// Get Vtk_ValHdl (if not Null)
	Vtk_ValHdl **hdl = NULL;
	if (pHdl)
		hdl = &(pHdl->m_pHdl);

	// Add individual certificate to Validation query
	Vtk_uint32 nRet = Vtk_ValidationAddCertRaw(context.GetVtkCtxt(), m_pValidation, 
		                                       &certBuffer, &issuerCertBuffer, hdl);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}




void CValidator::AddCertificateChain(const CContext& context, const CCertificate& cert,
								     Vtk_ChainBuildCallback callback /*=NULL*/, 
									 void *userHdl /*=NULL*/)
{
	assert(context.GetVtkCtxt());
	assert(cert.GetVtkCert());
	assert(m_pValidation);

	// Add individual certificate to Validation query
	Vtk_uint32 nRet = Vtk_ValidationAddCertChain(context.GetVtkCtxt(), m_pValidation, 
		                                         cert.GetVtkCert(), callback, userHdl);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}



void CValidator::Validate(const CContext& context, Vtk_uint32 *pValStatus, 
						  VString *pStrStatus) const
{
	assert(context.GetVtkCtxt());
	assert(m_pValidation);
	pStrStatus->erase();

	// Perform validation query on the certificates
	Vtk_uint32 nRet = Vtk_ValidationValidate(context.GetVtkCtxt(), m_pValidation, pValStatus);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}

	// Put status into the string
	char ** statusStrings;
	statusStrings = Vtk_StatusToStrings(*pValStatus);
	if (statusStrings)
	{
		char **temp = statusStrings;

		// Walk through the array of strings returned, printing each string,
		// until the end of array is reached - NULL entry.
		while (*temp)
		{
			*pStrStatus += *temp;
			*pStrStatus += "    ";
			temp++;
		}

		// Release status array.
		Vtk_StatusStringsDelete(statusStrings);
	}
}



void CValidator::SetVaInfo(const CContext& context, const VString& vaUrl, 
						   enum Vtk_ValidationMech mech, 
						   const CCertificateStore* pCertStore /*=NULL*/)
{
	assert(context.GetVtkCtxt());
	assert(m_pValidation);
	if (pCertStore)
		assert(pCertStore->GetVtkCertStore());

	// Get Vtk structures for Certificate Store (if not Null)
	const Vtk_CertStore *pVtkCertStore = NULL;
	if (pCertStore)
		pVtkCertStore = pCertStore->GetVtkCertStore();
	

	// Set VA info
	Vtk_uint32 nRet = Vtk_ValidationSetVaInfo(context.GetVtkCtxt(), m_pValidation,
											  vaUrl.c_str(), mech, pVtkCertStore);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}



void CValidator::GetRevStatus(const CContext& context, const CCertificate& cert,
						      const CCertificate& issuerCert, Vtk_uint32 *pStatus, 
							  CValRespDetails *pRespDetails /*=NULL*/,
							  CValRespSingleCertDetails *pCertDetails /*=NULL*/) const
{
	assert(context.GetVtkCtxt());
	assert(cert.GetVtkCert());
	assert(issuerCert.GetVtkCert());
	assert(m_pValidation);

	// Get Vtk_ValRespDetails for Vtk_CRLValidateCert (if not Null)
	Vtk_ValRespDetails **respDetails = NULL;
	if (pRespDetails)
		respDetails = &(pRespDetails->m_pRespDetails);

	// Get Vtk_ValRespDetails for Vtk_CRLValidateCert (if not Null)
	Vtk_ValRespSingleCertDetails **certDetails = NULL;
	if (pCertDetails)
		certDetails = &(pCertDetails->m_pCertDetails);

	// Retrive validation status information for a single certificate.
	Vtk_uint32 nRet = Vtk_ValidationGetRevStatus(context.GetVtkCtxt(), m_pValidation,
	                   cert.GetVtkCert(), issuerCert.GetVtkCert(), pStatus,
	                   respDetails, certDetails);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}



void CValidator::GetQueries(const CContext& context, CValQueries& valQueries) const
{
	assert(context.GetVtkCtxt());
	assert(m_pValidation);

	// Get Validation Queries
	Vtk_uint32 nRet = Vtk_ValidationGetQueries(context.GetVtkCtxt(), m_pValidation,
									&(valQueries.m_size), &(valQueries.m_ppQuery)); 
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}

	// Reset response data in all returned validation queries
	for (int i=0; i<valQueries.Size(); i++)
		valQueries.m_ppQuery[i]->response.dPtr = NULL;
}


void CValidator::ValidateFromQueries(const CContext& context, CValQueries& valQueries,
							Vtk_uint32 *pStatus, VString *pStrStatus)
{
	assert(context.GetVtkCtxt());
	assert(! valQueries.IsNull());
	assert(m_pValidation);


	// Performs validation of responses in valQueries
	Vtk_uint32 nRet = Vtk_ValidationValidateFromQueries(context.GetVtkCtxt(), m_pValidation,
	                             valQueries.m_ppQuery, pStatus);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}

	// Put status into the string
	char ** statusStrings;
	statusStrings = Vtk_StatusToStrings(*pStatus);
	if (statusStrings)
	{
		char **temp = statusStrings;

		// Walk through the array of strings returned, printing each string,
		// until the end of array is reached - NULL entry.
		while (*temp)
		{
			*pStrStatus += *temp;
			*pStrStatus += "    ";
			temp++;
		}

		// Release status array.
		Vtk_StatusStringsDelete(statusStrings);
	}
}



void CValidator::GetValHdl(const CContext& context, const CCertificate& cert,
	               const CCertificate *pIssuerCert, CValHdl *pHdl)
{
	assert(context.GetVtkCtxt());
	assert(cert.GetVtkCert());
	if (pIssuerCert)
		assert(pIssuerCert->GetVtkCert());
	assert(m_pValidation);
	assert(pHdl);


	// Get Vtk_ValHdl
	Vtk_ValHdl **hdl = &(pHdl->m_pHdl);

	// Get Vtk_Cert for issuer certificate
	const Vtk_Cert *pIssVtkCert = NULL;
	if (pIssuerCert)
		pIssVtkCert = pIssuerCert->GetVtkCert();


	// Get validation handle
	Vtk_uint32 nRet = Vtk_ValidationGetValHdl(context.GetVtkCtxt(), m_pValidation,
	                                          cert.GetVtkCert(), pIssVtkCert, hdl);
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}


void CValidator::AddReqExt(const CContext& context, const CExtension& ext)
{
	assert(context.GetVtkCtxt());
	assert(ext.GetVtkExt());
	assert(m_pValidation);


	// Adds extension to OCSP/CRT validation request
	Vtk_uint32 nRet = Vtk_ValidationAddReqExt(context.GetVtkCtxt(), m_pValidation, 
	                                          ext.GetVtkExt());
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}


void CValidator::AddReqExtForSingleCert(const CContext& context, const CCertificate& cert, 
		                        const CCertificate *pIssuerCert, const CExtension& ext)
{
	assert(context.GetVtkCtxt());
	assert(cert.GetVtkCert());
	if (pIssuerCert)
		assert(pIssuerCert->GetVtkCert());
	assert(ext.GetVtkExt());
	assert(m_pValidation);


	// Get Vtk_Cert for issuer certificate
	const Vtk_Cert *pIssVtkCert = NULL;
	if (pIssuerCert)
		pIssVtkCert = pIssuerCert->GetVtkCert();

	// Adds extension to OCSP validation request for particular certificate
	Vtk_uint32 nRet = Vtk_ValidationAddReqExtForSingleCert(context.GetVtkCtxt(), 
		               m_pValidation, cert.GetVtkCert(), pIssVtkCert, ext.GetVtkExt());
	if (nRet != VTK_OK)
	{
		VString error(Vtk_ErrorToString(nRet));
		throw error;
	}
}
